//
// CONFIG FILE
//
ArrayList types = new ArrayList();
void readConfigFile( String file )
{
  println( "Reading config file..." );

  String[] rows = loadStrings( file );

  int counter = 0;
  String[] cols;
  ArrayList region_files = new ArrayList();
  
  boolean normalize_globally = false;

  // read in the lines and store the file names
  while ( counter < rows.length )
  {
    cols = splitTokens( rows[counter++] );  

    if ( cols.length == 0 )
      continue;

    else if ( cols[0].equals("REGION:") )
      region_files.add( cols[1] );
      
    else if ( cols[0].equals("REGION_LABEL:") )
      _region_label = cols[1];
      
    else if ( cols[0].equals("TYPE_LABEL:") )
      _type_label = cols[1];
      
    else if ( cols[0].equals("NORMALIZATION:") ) {
      if ( cols[1].equalsIgnoreCase("global") ) normalize_globally = true;
    }
    
    else if ( cols[0].equals("GLOBAL_MIN_VALUE:") )
      _global_min_v = parseFloat(cols[1]);
  }

  // read in the region files
  _regions = new Region[region_files.size()];
  for ( int i = 0; i < region_files.size(); i++ )
    _regions[i] = readRegionFile( (String)region_files.get(i) );

  print( "    regions: " );
  print( _regions[0].tag() );
  for ( int i = 1; i < region_files.size(); i++ )
    print( ", " + _regions[i].tag() );
  println();

  // store the binding site types
  _binding_site_types = new BindingSiteType[types.size()];
  _binding_site_order = new int[types.size()];
  for ( int i = 0; i < types.size(); i++ ) {
    _binding_site_types[i] = new BindingSiteType( (String)types.get(i) );
    _binding_site_order[i] = i;
  }

  print( "    binding site types: " );
  print( _binding_site_types[0].tag() );
  for ( int i = 0; i < _binding_site_types.length; i++ )
    print( ", " + _binding_site_types[i].tag() );
  println();
  
  // determine the min and max strength values
  determineMinMaxStrengths( normalize_globally );
}

//
// REGION FILE
//
Region readRegionFile( String file )
{
  String[] rows = loadStrings( file );

  int counter = 0;
  String[] cols;

  Region region = new Region();
  while ( counter < rows.length )
  {
    cols = splitTokens( rows[counter++], ", :" );  

    if ( cols.length == 0 )
      continue;

    else if ( cols[0].substring(0,2).equals("##") )
      continue;

    else if ( cols[0].equalsIgnoreCase("#NAME") )
      region.tag( cols[1] );

    else if ( cols[0].equalsIgnoreCase("#START") )
      region.start( Long.parseLong( cols[1]) );

    else if ( cols[0].equalsIgnoreCase("#LENGTH") )
      region.len( Long.parseLong( cols[1] ) );

    else if ( cols[0].equalsIgnoreCase("binding_site") )
    {
      // first, determine the type
      int t = -1;

      // look for this type in the existing list
      for ( int i = 0; i < types.size(); i++ )
      {
        if ( cols[3].equals( (String)types.get(i) ) ) 
          t = i;
      }

      // if this type doesn't exist, add it
      if ( t == -1 )
      {
        types.add( cols[3] );
        t = types.size()-1;
      }

      // check for a strength
      float s = 1.0;
      if ( cols.length == 5 ) s = Float.parseFloat( cols[4] );

      // add a new binding site
      region.addBindingSite( new BindingSite( Integer.parseInt(cols[1]), Integer.parseInt(cols[2]), t, s ) );
    }
    
    else if ( cols[0].equalsIgnoreCase("feature") || cols[0].equalsIgnoreCase("annotation") )
    {
      String tag = "";
      if ( cols.length == 4 ) tag = cols[3];
      region.addFeature( new Feature( Integer.parseInt(cols[1]), Integer.parseInt(cols[2]), tag ) );
    }
  }

  region.computeStop();
  region.addStartStopFeatures();
  region.setRelativeStartsAndStops();
  region.sortBindingSites();

  return region;
}

