//----------------------------------------------------
//
class BindingSiteType
{
  String _tag;
  boolean _show;
  color _show_color;
  float _min_strength, _max_strength;
  
  BindingSiteType( String tag )
  {
    _tag = tag;
        
    _show = true;
    _show_color = DEFAULT_COLOR;
  }
  
  String tag() { return _tag; }  
  
  void show( boolean s ) { _show = s; }
  void showColor( color c ) { _show_color = c; }
  boolean show() { return _show; }
  color showColor() { return _show_color; }
  
  void minMaxStrength( float min_s, float max_s ) { _min_strength = min_s; _max_strength = max_s; }
  float minStrength() { return _min_strength; }
  float maxStrength() { return _max_strength; }
  float maxMinusMinStrength() { return _max_strength-_min_strength; }
}


//----------------------------------------------------
//
class Feature extends BasicRegion
{
  Feature( long start, long len, String tag )
  {
    super( start, len, tag );
  }
}


//----------------------------------------------------
//
class BindingSite extends BasicRegion
{
  int _type;
  float _strength;
  
  
  BindingSite( long start, long len, int type, float strength )
  {
    super( start, len, "" );
    
    _type = type;
    _strength = strength;
  }
  
  int type() { return _type; }
  float strength() { return _strength; }
}


//----------------------------------------------------
//
class Region extends BasicRegion
{
  ArrayList _binding_sites, _features;
  float _pixel_per_bp;
  int _render_start, _render_stop;
  boolean _show;
  
  Region()
  {
    _binding_sites = new ArrayList();
    _features = new ArrayList();
    _show = true;
  }
  
  Region( long start, long len, String tag )
  {
    super( start, len, tag );
    
    _binding_sites = new ArrayList();
    _features = new ArrayList();
    _show = true;
  }
  
  void addBindingSite( BindingSite bs ) { _binding_sites.add(bs); }
  void addFeature( Feature f ) { _features.add(f); }
  
  void sortBindingSites() {
    BindingSiteComparator1 comparator1 = new BindingSiteComparator1();
    //BindingSiteComparator2 comparator2 = new BindingSiteComparator2();
    Collections.sort( _binding_sites, comparator1 );
    //Collections.sort( _binding_sites, comparator2 );
  }
  
  void show( boolean s ) { _show = s; }
  boolean show() { return _show; }
  
  BindingSite bindingSite( int i ) { return (BindingSite)_binding_sites.get(i); }
  Feature feature( int i ) { return (Feature)_features.get(i); }
  
  int numBindingSites() { return _binding_sites.size(); }
  int numFeatures() { return _features.size(); }
  
  void pixelPerBP( float v ) { _pixel_per_bp = v; }
  float pixelPerBP() { return _pixel_per_bp; }
  void renderStartStop( int start, int stop ) { _render_start = start; _render_stop = stop; }
  int renderStart() { return _render_start; }
  int renderStop() { return _render_stop; }
  
  void setRelativeStartsAndStops()
  {
    for ( int i = 0; i < numBindingSites(); i++ )
      bindingSite(i).computeRelativeStartStop( _start );
    
    for ( int i = 0; i < numFeatures(); i++ )
      feature(i).computeRelativeStartStop( _start );
  }
  
  void addStartStopFeatures()
  {
    addFeature( new Feature( _start, 0, "") );
    addFeature( new Feature( _stop, 0, "") );
  }
}

public class BindingSiteComparator1 implements Comparator<BindingSite> {
  public int compare(BindingSite bs1, BindingSite bs2) {
    return ((Long)(bs2._len)).compareTo((Long)(bs1._len));
  }
}
  
public class BindingSiteComparator2 implements Comparator<BindingSite> {
  public int compare(BindingSite bs1, BindingSite bs2) {
    return ((Float)(bs2._strength)).compareTo((Float)(bs1._strength));
  }
}

//----------------------------------------------------
//
// the basic definition of a genomic region
//
class BasicRegion
{
  long _start, _stop, _len;
  long _relative_start, _relative_stop;
  String _tag;
  
  BasicRegion() {}
  
  BasicRegion( long start, long len, String tag )
  {
    _start = start;
    _len = len;
    _tag = tag;
    computeStop();
  }
  
  void tag( String t ) { _tag = t; }
  void start( long s ) { _start = s; }
  void len( long l ) { _len = l; }
  
  long start() { return _start; }
  long len() { return _len; }
  long stop() { return _stop; }
  String tag() { return _tag; }
  long relativeStart() { return _relative_start; }
  
  void computeStop() { _stop = _start + _len - 1; }
  
  void computeRelativeStartStop( long start )
  {
    _relative_start = _start - start;
    _relative_stop = _stop - start;
  }
}

