class RegionViewer
{
  int _width, _height;
  int _regions_view_w, _regions_view_h, _regions_w, _regions_h;
  int[] _regions_origin;
  int _alignment_selector_w, _alignment_selector_h;
  int[] _alignment_selector_origin;
  int _site_legend_w, _site_legend_h;
  int[] _site_legend_origin;
  int _spatial_legend_w, _spatial_legend_h;
  int[] _spatial_legend_origin;
  int _color_picker_w, _color_picker_h;
  int[] _color_picker_origin;
  int _bar_height_w, _bar_height_h;
  int[] _bar_height_origin;

  int _region_alignment;
  int _region_tag_max_width, _binding_site_type_tag_max_width;
  int[] _alignment_selector_line_w;
  int _max_alignment_selector_line_w;
  int _num_active_binding_sites;
  boolean _color_picker_one_row;

  boolean _binding_site_rolled_over, _feature_rolled_over;
  int _rolled_over_binding_site, _rolled_over_feature, _rolled_over_region;

  boolean _color_selected, _legend_site_color_selected, _clear_button_selected;
  int _selected_color, _selected_legend_site_color;
  int[] _color_selected_box_offset, _legend_site_color_box_offset, _legend_site_color_box;

  boolean _binding_site_type_rolled_over;
  int _rolled_over_binding_site_type;

  int _scale_pixel_length;
  String _scale_tag;

  boolean _site_types_dropdown_pressed, _site_types_dropped_down;
  boolean _regions_dropdown_pressed, _regions_dropped_down;
  int _site_types_dropbox_w, _site_types_dropbox_h, _regions_dropbox_w, _regions_dropbox_h;
  int[] _site_types_dropbox_origin, _regions_dropbox_origin;
  boolean _site_types_dropdown_rolled_over, _regions_dropdown_rolled_over;
  int _rolled_over_site_types_dropdown, _rolled_over_regions_dropdown;
  boolean _site_types_dropbox_pressed, _site_types_dropbox_highlighted, _regions_dropbox_pressed, _regions_dropbox_highlighted;
  int _pressed_site_types_dropbox, _pressed_regions_dropbox;

  int _total_legend_h;

  float _percent_regions_shown, _percent_regions_start;
  int _y_scroll_bar_h, _y_scroll_bar_start;
  boolean _y_scroll_present, _y_scroll_bar_rolled_over, _y_scroll_bar_pressed;
  int _y_scroll_bar_pressed_y;

  RegionViewer()
  {
    _regions_origin = new int[2];   
    _alignment_selector_origin = new int[2];
    _site_legend_origin = new int[2];
    _spatial_legend_origin = new int[2];
    _color_picker_origin = new int[2];
    _color_selected_box_offset = new int[2];
    _legend_site_color_box_offset = new int[2];
    _legend_site_color_box = new int[2];
    _site_types_dropbox_origin = new int[2];
    _regions_dropbox_origin = new int[2];
    _bar_height_origin = new int[2];

    _region_alignment = REGION_ALIGNMENT_LEFT;
    _binding_site_rolled_over = _feature_rolled_over = _binding_site_type_rolled_over = false;
    _color_selected = _legend_site_color_selected = _clear_button_selected = false; 
    _site_types_dropdown_pressed = _site_types_dropped_down = false;
    _regions_dropdown_pressed = _regions_dropped_down = false;
    _site_types_dropdown_rolled_over = _regions_dropdown_rolled_over = false;
    _site_types_dropdown_pressed = _regions_dropdown_pressed = false;
    _site_types_dropbox_highlighted = _regions_dropbox_highlighted = false;

    _alignment_selector_line_w = new int[5];
    _alignment_selector_line_w[0] = _alignment_selector_line_w[4] = 10;
    _alignment_selector_line_w[1] = _alignment_selector_line_w[3] = 6;
    _alignment_selector_line_w[2] = 8;
    _max_alignment_selector_line_w = 10;
    _alignment_selector_w = 4*ALIGNMENT_SELECTOR_BOX_W;
    _alignment_selector_h = ALIGNMENT_SELECTOR_BOX_W;

    // find the max pixel width of the region and binding site type tags
    _region_tag_max_width = _binding_site_type_tag_max_width = Integer.MIN_VALUE;
    textFont( _pixel_font_8b );
    for ( int i = 0; i < _binding_site_types.length; i++ )
      _binding_site_type_tag_max_width = max( _binding_site_type_tag_max_width, (int)textWidth( _binding_site_types[i].tag() ) );
    for ( int i = 0; i < _regions.length; i++ )
      _region_tag_max_width = max( _region_tag_max_width, (int)textWidth( _regions[i].tag() ) );

    _site_types_dropbox_w = 2*DROPBOX_BORDER + DROPBOX_RADIO_BUTTON_W + DROPBOX_RADIO_BUTTON_OFFSET + _binding_site_type_tag_max_width;
    _site_types_dropbox_h = 2*DROPBOX_BORDER + _binding_site_types.length*(DROPBOX_LINE_H+DROPBOX_LINE_OFFSET) - DROPBOX_LINE_OFFSET;
    _site_types_dropbox_origin[0] = _site_types_dropbox_origin[1] = 0;

    // set the site legend width
    _site_legend_w = 2*BINDING_SITE_LEGEND_BORDER_W + BINDING_SITE_LEGEND_BOX_W + BINDING_SITE_LEGEND_BOX_OFFSET + _binding_site_type_tag_max_width;
    _site_legend_w = 100; 

    _spatial_legend_w = _site_legend_w;  
    _bar_height_w = _site_legend_w;

    _regions_dropbox_w = 2*DROPBOX_BORDER + DROPBOX_RADIO_BUTTON_W + DROPBOX_RADIO_BUTTON_OFFSET + _region_tag_max_width;
    _regions_dropbox_h = 2*DROPBOX_BORDER + _regions.length*(DROPBOX_LINE_H+DROPBOX_LINE_OFFSET) - DROPBOX_LINE_OFFSET;
    textFont( _verdana_font_12 );
    _region_tag_max_width = max( _region_tag_max_width, (int)textWidth(_region_label)+DROP_DOWN_BOX_W+DROP_DOWN_BOX_OFFSET ) + REGION_TAG_OFFSET;
    _regions_dropbox_origin[0] = /*_spatial_legend_w - _regions_dropbox_w*/ -_region_tag_max_width;
    _regions_dropbox_origin[1] = 0 /*4*/;

    _spatial_legend_h = _regions_dropbox_h + 5;
    _bar_height_h = 30;

    // set the color picker height and width (assume 1 row of color boxes and then test
    _color_picker_h = COLOR_PICKER_BOX_W + COLOR_PICKER_BUTTON_OFFSET + CLEAR_BUTTON_H;
    _color_picker_w = NUM_SITE_COLORS*COLOR_PICKER_BOX_W + (NUM_SITE_COLORS-1)*COLOR_PICKER_BOX_OFFSET;
    _color_picker_one_row = true;
    if ( _site_legend_w < _color_picker_w )
    {
      _color_picker_w -= ((NUM_SITE_COLORS/2)*COLOR_PICKER_BOX_W + (NUM_SITE_COLORS/2)*COLOR_PICKER_BOX_OFFSET);
      _color_picker_one_row = false;
      _color_picker_h += (COLOR_PICKER_BOX_W + COLOR_PICKER_BOX_OFFSET);
    }

    _site_legend_w = max( _site_legend_w, _color_picker_w );

    _regions_origin[0] = _region_tag_max_width - 1;
    _regions_origin[1] = SCALE_BAR_OFFSET;
    _alignment_selector_origin[1] = _regions_origin[1] + 5;
    _bar_height_origin[1] = _alignment_selector_origin[1] + _alignment_selector_h + ALIGNMENT_SELECTOR_OFFSET + BINDING_SITE_LEGEND_TEXT_OFFSET + 20;
    _site_legend_origin[1] = _bar_height_origin[1] + _bar_height_h + ALIGNMENT_SELECTOR_OFFSET + BINDING_SITE_LEGEND_TEXT_OFFSET + 20;

    _num_active_binding_sites = _binding_site_types.length;
    _site_legend_h = 2*BINDING_SITE_LEGEND_BORDER_W + _num_active_binding_sites*BINDING_SITE_LEGEND_BOX_H + 
      (_num_active_binding_sites-1)*BINDING_SITE_LEGEND_BOX_OFFSET;
    _spatial_legend_origin[1] = _site_legend_origin[1] + _site_legend_h + ALIGNMENT_SELECTOR_OFFSET + BINDING_SITE_LEGEND_TEXT_OFFSET + 20 + 
      _color_picker_h + LEGEND_COLOR_PICKER_OFFSET;

    _color_picker_origin[1] = _site_legend_origin[1] + _site_legend_h + LEGEND_COLOR_PICKER_OFFSET;

    // compute the site legend height (which eventually may use the color picker info
    computeLegendParameters();

    _total_legend_h = _color_picker_origin[1] + _color_picker_h;

    // compute the total height of the regions
    computeTotalRegionsHeight();

    _percent_regions_start = 0.0;
    _y_scroll_bar_start = 0;
    _y_scroll_bar_rolled_over = _y_scroll_bar_pressed = false;
  }

  void computeTotalRegionsHeight() {
    _regions_h = 0;

    for ( int i = 0; i < _regions.length; i++ ) {
      if ( _regions[i].show() ) _regions_h += REGION_HEIGHT + REGION_OFFSET;
    }
    if ( _regions_h != 0 ) _regions_h -= REGION_OFFSET;

    computeYScrollBarDimensions();
  }

  void computeYScrollBarDimensions() {
    if ( _regions_h > _regions_view_h ) {

      _percent_regions_shown = (float)_regions_view_h / (float)_regions_h;
      _y_scroll_bar_h = (int)(_percent_regions_shown * _regions_view_h);

      //check if we are going from not scrolling to scrolling
      if ( !_y_scroll_present ) {
        _y_scroll_bar_start = 0;
        _percent_regions_start = 0.0;
      }
      else { 
        if ( (_percent_regions_shown+_percent_regions_start) > 1.0 ) {
          _y_scroll_bar_start = _regions_view_h - _y_scroll_bar_h; 
          _percent_regions_start = (float)_y_scroll_bar_start / (float)_regions_view_h;
        }
        else {
          _y_scroll_bar_start = (int)(_percent_regions_start*_regions_view_h);
        }
      }

      _y_scroll_present = true;
    }
    else {
      _percent_regions_start = 0.0;
      _percent_regions_shown = 1.0;
      _y_scroll_present = false;
    }
  }

  // this will need to be called when the number of binding sites change
  void computeLegendParameters()
  {
    // count the number of active binding site types
    _num_active_binding_sites = 0;
    for ( int i = 0; i < _binding_site_types.length; i++ )
      if ( _binding_site_types[i].show() ) ++_num_active_binding_sites;

    // compute the site legend height
    _site_legend_h = 2*BINDING_SITE_LEGEND_BORDER_W + _num_active_binding_sites*BINDING_SITE_LEGEND_BOX_H + 
      (_num_active_binding_sites-1)*BINDING_SITE_LEGEND_BOX_OFFSET;

    _color_picker_origin[1] = _site_legend_origin[1] + _site_legend_h + LEGEND_COLOR_PICKER_OFFSET;
  }

  // this is called when the window size changes
  void setDimensions( int w, int h )
  {
    _width = w;
    _height = h;

    _site_legend_origin[0] = _alignment_selector_origin[0] = _bar_height_origin[0] = w - max(_site_legend_w, _alignment_selector_w) - 1; 
    _color_picker_origin[0] = _site_legend_origin[0]+BINDING_SITE_LEGEND_BORDER_W;  
    _spatial_legend_origin[0] = _site_legend_origin[0];

    _regions_w = _regions_view_w = _site_legend_origin[0] - BINDING_SITE_LEGEND_OFFSET - _regions_origin[0];
    _regions_view_h = h - _regions_origin[1];

    setAlignmentAndScaleParameters();
    computeYScrollBarDimensions();
  }

  // this needs to be called whenever the regions change
  void setAlignmentAndScaleParameters()
  {   
    // get the max length for just the regions that are being shown 
    determineMaxRegionLength();

    // set the bp/pixel and regions start/stops
    float pixel_per_bp = (float)_regions_view_w / (float)_regions_max_len;
    switch ( _region_alignment )
    {
    case REGION_ALIGNMENT_LEFT:

      for ( int i = 0; i < _regions.length; i++ )
      {
        _regions[i].pixelPerBP( pixel_per_bp );
        _regions[i].renderStartStop( 0, round( (float)(_regions[i].len()-1)*pixel_per_bp ) );
      }
      break;

    case REGION_ALIGNMENT_RIGHT:

      for ( int i = 0; i < _regions.length; i++ )
      {
        _regions[i].pixelPerBP( pixel_per_bp );
        _regions[i].renderStartStop( _regions_view_w-round((float)_regions[i].len()*pixel_per_bp), _regions_view_w-1  );
      }
      break;

    case REGION_ALIGNMENT_CENTER:

      for ( int i = 0; i < _regions.length; i++ )
      {
        _regions[i].pixelPerBP( pixel_per_bp );
        int diff = _regions_view_w-round((float)_regions[i].len()*pixel_per_bp);
        _regions[i].renderStartStop( diff/2, _regions_view_w-1-diff/2  );
      }
      break;

    case REGION_ALIGNMENT_STRETCH:

      for ( int i = 0; i < _regions.length; i++ )
      {
        _regions[i].pixelPerBP( (float)_regions_view_w / (float)_regions[i].len()  );
        _regions[i].renderStartStop( 0, (_regions_view_w-1) );
      }
      break;
    }

    // set the scale bar parameters
    long scale_length = 10000000; 
    _scale_tag = "10 Mbp";
    if ( _regions_max_len <= 20 ) { 
      scale_length = 1; 
      _scale_tag = "1 bp";
    }
    else if ( _regions_max_len <= 200 ) { 
      scale_length = 10; 
      _scale_tag = "10 bp";
    }
    else if ( _regions_max_len <= 2000 ) { 
      scale_length = 100; 
      _scale_tag = "100 bp";
    }
    else if ( _regions_max_len <= 20000 ) { 
      scale_length = 1000; 
      _scale_tag = "1 kbp";
    }
    else if ( _regions_max_len <= 200000 ) { 
      scale_length = 10000; 
      _scale_tag = "10 kbp";
    }
    else if ( _regions_max_len <= 2000000 ) { 
      scale_length = 100000; 
      _scale_tag = "100 kbp";
    }
    else if ( _regions_max_len <= 20000000 ) { 
      scale_length = 1000000; 
      _scale_tag = "1 Mbp";
    }
    _scale_pixel_length = round((float)scale_length*pixel_per_bp);
  }

  void render()
  {
    int x, y, w, h;

    //
    // render the alignment selector
    //
    pushMatrix();
    translate( _alignment_selector_origin[0], _alignment_selector_origin[1] );

    x = y = 0;

    // the alignment label
    //    textFont( _verdana_font_12 );
    //    textAlign( LEFT, BOTTOM );
    //    fill( LABEL_COLOR );
    //    text( "justify", x/*+BINDING_SITE_LEGEND_BORDER_W*/, y-4 );

    stroke( REGION_LINE_COLOR );
    strokeWeight( 1 );
    line( x, y-5, x+_site_legend_w, y-5 );
    fill( REGION_LINE_COLOR );
    textFont( _pixel_font_8 );
    textAlign( RIGHT, BOTTOM );
    text( "justify", x+_site_legend_w, y-7 );


    for ( int i = REGION_ALIGNMENT_LEFT; i <= REGION_ALIGNMENT_STRETCH; i++ )
    {
      renderAnAlignmentSelector( i, x, y );
      x += ALIGNMENT_SELECTOR_BOX_W;
    }

    popMatrix();

    //
    // render the bar height legend
    //
    pushMatrix();
    translate( _bar_height_origin[0], _bar_height_origin[1] );

    x = y = 0;

    stroke( REGION_LINE_COLOR );
    strokeWeight( 1 );
    line( x, y-20, x+_bar_height_w, y-20 );
    fill( REGION_LINE_COLOR );
    textFont( _pixel_font_8 );
    textAlign( RIGHT, BOTTOM );
    text( "bar height", x+_site_legend_w, y-22 );

    // the legend label
    //    textFont( _verdana_font_12 );
    //    textAlign( LEFT, BOTTOM );
    //    fill( LABEL_COLOR );
    //    text( "strength", x/*+BINDING_SITE_LEGEND_BORDER_W*/, y-4 );

    // the bar legend
    x += LEGEND_COLOR_PICKER_OFFSET;
    y -= 5;
    int rh = 40;
    int rw = 26;
    rectMode( CORNER );
    noStroke();
    fill( REGION_MAX_VALUE_BAND_COLORS[0] );
    rect( x, y, rw, rh/2 );
    fill( REGION_MAX_VALUE_BAND_COLORS[1] );
    rect( x, y+rh/2, rw, rh/2 );

    strokeWeight( REGION_LINE_WEIGHT );
    stroke( REGION_HALF_VALUE_LINE_COLOR );
    line( x, y+rh/2, x+rw, y+rh/2 );

    //    noStroke();
    //    fill( DEFAULT_COLOR );
    //    rect( x+8, y, 10, rh );

    stroke( DEFAULT_COLOR );
    line( x, y, x+rw+10, y );
    line( x, y+rh, x+rw+10, y+rh );

    if ( _binding_site_rolled_over || _binding_site_type_rolled_over )
    {
      float max_s;
      float min_s;
      if ( _binding_site_type_rolled_over ) {
        max_s = _binding_site_types[_binding_site_order[_rolled_over_binding_site_type]].maxStrength();
        min_s = _binding_site_types[_binding_site_order[_rolled_over_binding_site_type]].minStrength();
      }
      else {
        max_s = _binding_site_types[_regions[_rolled_over_region].bindingSite(_rolled_over_binding_site).type()].maxStrength();
        min_s = _binding_site_types[_regions[_rolled_over_region].bindingSite(_rolled_over_binding_site).type()].minStrength();
      }

      textFont( _pixel_font_8 );
      fill( LABEL_COLOR );

      textAlign( LEFT, BOTTOM );
      text( max_s, x+rw+12, y+1 );

      textAlign( LEFT, TOP );
      text( min_s, x+rw+12, y+rh-3 );

      // if a binding site is rolled over, show it in the legend
      if ( _binding_site_rolled_over ) {
        float value = _regions[_rolled_over_region].bindingSite(_rolled_over_binding_site).strength();
        y += round((1.0 - (value - min_s)/(max_s - min_s)) * rh);

        stroke( ROLLED_OVER_BINDING_SITE_LINE_COLOR );
        line( x+8, y, x+rw+10, y );
        fill( DEFAULT_COLOR );
        rect( x+8, y, 10, rh-y-5 );
        textAlign( LEFT, CENTER );
        fill( LABEL_COLOR );
        text( value, x+rw+12, y );
      }
    }



    popMatrix();



    //
    // render the color picker
    //
    pushMatrix();
    translate( _color_picker_origin[0], _color_picker_origin[1] );

    // render the color choices
    x = 0;
    y = 0;
    noStroke();

    for ( int i = 0; i < (NUM_SITE_COLORS/2); i++ )
    { 
      fill( SITE_COLOR[i] );
      rect( x, y, COLOR_PICKER_BOX_W, COLOR_PICKER_BOX_W );
      x += (COLOR_PICKER_BOX_W + COLOR_PICKER_BOX_OFFSET );
    }

    if ( !_color_picker_one_row )
    {
      x = 0;
      y += (COLOR_PICKER_BOX_W + COLOR_PICKER_BOX_OFFSET );
    }

    for ( int i = (NUM_SITE_COLORS/2); i < NUM_SITE_COLORS; i++ )
    { 
      fill( SITE_COLOR[i] );
      rect( x, y, COLOR_PICKER_BOX_W, COLOR_PICKER_BOX_W );
      x += (COLOR_PICKER_BOX_W + COLOR_PICKER_BOX_OFFSET );
    }

    // render the clear button
    x = 0;
    y += (COLOR_PICKER_BOX_W + COLOR_PICKER_BUTTON_OFFSET );
    if ( !_clear_button_selected )
    {
      fill( COLOR_PICKER_BUTTON_FILL_COLOR );
      stroke( COLOR_PICKER_LINE_COLOR );
    }
    else
    {
      fill( COLOR_PICKER_BUTTON_SELECTED_FILL_COLOR );
      stroke( COLOR_PICKER_SELECTED_LINE_COLOR );
    }
    textFont( _pixel_font_8 );
    rect( x, y, textWidth("clear")+4, CLEAR_BUTTON_H );
    if ( !_clear_button_selected ) fill( COLOR_PICKER_BUTTON_TEXT_COLOR );
    else fill ( COLOR_PICKER_SELECTED_LINE_COLOR );
    textAlign( LEFT, CENTER );
    text( "clear", x+3, y+COLOR_PICKER_BOX_W/2-1 );

    popMatrix();

    //
    // render the site legend
    //
    pushMatrix();
    translate( _site_legend_origin[0], _site_legend_origin[1] );

    // render the site types
    x = BINDING_SITE_LEGEND_BORDER_W;
    y = BINDING_SITE_LEGEND_BORDER_W;

    BindingSiteType binding_site_type;
    textAlign( LEFT, CENTER );
    strokeWeight( 1 );
    int type_rolled_over_binding_site = -1;
    if ( _binding_site_rolled_over ) 
      type_rolled_over_binding_site = _regions[_rolled_over_region].bindingSite(_rolled_over_binding_site).type();
    else if ( _binding_site_type_rolled_over )
      type_rolled_over_binding_site = _rolled_over_binding_site_type;
    for ( int i = 0; i < _binding_site_types.length; i++ )
    {
      binding_site_type = _binding_site_types[_binding_site_order[i]];

      if ( !binding_site_type.show() ) continue;

      if ( _legend_site_color_selected && (_selected_legend_site_color == i) ) fill( DEFAULT_COLOR );
      else fill( binding_site_type.showColor() );
      if ( type_rolled_over_binding_site == i ) stroke( COLOR_PICKER_SELECTED_LINE_COLOR );
      else noStroke();
      rect( x, y, BINDING_SITE_LEGEND_BOX_W, BINDING_SITE_LEGEND_BOX_H );

      if ( type_rolled_over_binding_site == i ) 
      {
        textFont( _pixel_font_8b );
        fill( COLOR_PICKER_SELECTED_LINE_COLOR );
      }
      else
      {
        textFont( _pixel_font_8 );
        fill( BINDING_SITE_LEGEND_TEXT_COLOR );
      }
      text( binding_site_type.tag(), x+BINDING_SITE_LEGEND_BOX_OFFSET+BINDING_SITE_LEGEND_BOX_W, y+BINDING_SITE_LEGEND_BOX_H/2-2 );

      y += BINDING_SITE_LEGEND_BOX_OFFSET+BINDING_SITE_LEGEND_BOX_H;
    }

    // render the legend box
    x = y = 0;

    stroke( REGION_LINE_COLOR );
    strokeWeight( 1 );
    line( x, y-20, x+_site_legend_w, y-20 );
    fill( REGION_LINE_COLOR );
    textFont( _pixel_font_8 );
    textAlign( RIGHT, BOTTOM );
    text( "color", x+_site_legend_w, y-22 );

    // the legend label
    textFont( _verdana_font_12 );
    textAlign( LEFT, BOTTOM );
    fill( LABEL_COLOR );
    text( _type_label, x/*+BINDING_SITE_LEGEND_BORDER_W*/, y-4 );

    x += textWidth(_type_label) + DROP_DOWN_BOX_OFFSET;
    y -= DROP_DOWN_BOX_W-2;
    noStroke();
    fill( DROP_DOWN_FILL_COLOR );
    if ( !_site_types_dropped_down ) 
      triangle( x+DROP_DOWN_BOX_W/2, y, x+DROP_DOWN_BOX_W, y-DROP_DOWN_BOX_W, x, y-DROP_DOWN_BOX_W );
    else
    {
      triangle( x, y, x+DROP_DOWN_BOX_W, y, x+DROP_DOWN_BOX_W/2, y-DROP_DOWN_BOX_W );

      // render the drop down box
      x = _site_types_dropbox_origin[0];
      y = _site_types_dropbox_origin[1];
      fill( DROPBOX_FILL_COLOR );
      stroke( DROPBOX_LINE_COLOR );
      strokeWeight( 1 );
      rect( x, y, _site_types_dropbox_w, _site_types_dropbox_h );

      x += DROPBOX_BORDER;
      y += DROPBOX_BORDER + DROPBOX_LINE_H;
      textAlign( LEFT, BOTTOM );
      fill( DROPBOX_TEXT_COLOR );
      stroke( DROPBOX_TEXT_COLOR );
      for ( int i = 0; i < _binding_site_types.length; i++ )
      {
        binding_site_type = _binding_site_types[_binding_site_order[i]];

        if ( _site_types_dropbox_highlighted && (_pressed_site_types_dropbox == i) )
        {
          fill( HIGHLIGHTED_COLOR );
          stroke( HIGHLIGHTED_COLOR );
        }
        else
        {
          fill( DROPBOX_TEXT_COLOR );
          stroke( DROPBOX_TEXT_COLOR );
        }

        if ( _site_types_dropdown_rolled_over && (_rolled_over_site_types_dropdown == i) )
        {
          textFont( _pixel_font_8b );
          strokeWeight( 2 );
        }
        else
        {
          textFont( _pixel_font_8 );
          strokeWeight( 1 );
        }
        text( binding_site_type.tag(), x+DROPBOX_RADIO_BUTTON_W+DROPBOX_RADIO_BUTTON_OFFSET, y+2 );

        if ( binding_site_type.show() )
        {
          smooth();
          line( x+DROPBOX_RADIO_BUTTON_W/4, y-DROPBOX_RADIO_BUTTON_W/2, x+DROPBOX_RADIO_BUTTON_W/2, y );
          line( x+DROPBOX_RADIO_BUTTON_W/2, y, x+DROPBOX_RADIO_BUTTON_W, y-DROPBOX_RADIO_BUTTON_W );
          noSmooth();
        }

        y += DROPBOX_LINE_H + DROPBOX_LINE_OFFSET;
      }
    }

    popMatrix();

    //
    // render the regions
    //
    pushMatrix();
    translate( _regions_origin[0], _regions_origin[1] );

    Region region;

    int amt_scrolled = (int)(_percent_regions_start*_regions_h);
    pushMatrix();
    translate( 0, -amt_scrolled );

    int num_regions_skipped = floor( (float)amt_scrolled / (float)(REGION_HEIGHT+REGION_OFFSET) );
    int num_regions_ignored = max(0, floor( (float)(_regions_h - (amt_scrolled+_regions_view_h)) / (float)(REGION_HEIGHT+REGION_OFFSET) ));

    y = num_regions_skipped*(REGION_HEIGHT+REGION_OFFSET) + BINDING_SITE_HEIGHT;
    BindingSite binding_site;
    Feature feature;
    int rolled_bs_x=0, rolled_bs_w=0, rolled_bs_h=0;
    for ( int i = num_regions_skipped; i < _regions.length-num_regions_ignored; i++ )
    {
      region = _regions[i];

      if ( !region.show() ) continue;

      // render the background strength bars
      rectMode( CORNERS );
      noStroke();
      fill( REGION_MAX_VALUE_BAND_COLORS[0] );
      rect( region.renderStart(), y-BINDING_SITE_HEIGHT, region.renderStop(), y-BINDING_SITE_HEIGHT/2 );
      fill( REGION_MAX_VALUE_BAND_COLORS[1] );
      rect( region.renderStart(), y-BINDING_SITE_HEIGHT/2, region.renderStop(), y );

      strokeWeight( REGION_LINE_WEIGHT );
      stroke( REGION_HALF_VALUE_LINE_COLOR );
      line( region.renderStart(), y-BINDING_SITE_HEIGHT/2-1, region.renderStop(), y-BINDING_SITE_HEIGHT/2-1 );
      stroke( REGION_MAX_VALUE_LINE_COLOR );
      line( region.renderStart(), y-BINDING_SITE_HEIGHT, region.renderStop(), y-BINDING_SITE_HEIGHT );
      line( region.renderStart(), y-BINDING_SITE_HEIGHT, region.renderStart(), y );
      line( region.renderStop(), y-BINDING_SITE_HEIGHT, region.renderStop(), y );


      // render the noncolored, not rolled over binding sites
      rectMode( CORNER );
      for ( int j = 0; j < region.numBindingSites(); j++ )
      {
        binding_site = region.bindingSite(j);
        binding_site_type = _binding_site_types[binding_site.type()];

        if ( !binding_site_type.show() ) continue;        
        if ( _binding_site_type_rolled_over && (_rolled_over_binding_site_type == binding_site.type()) ) continue;
        if ( !(binding_site_type.showColor() == DEFAULT_COLOR) ) continue;

        // first, render the filled in binding site
        noStroke();
        fill( binding_site_type.showColor() );
        stroke( BINDING_SITE_OUTLINE_COLOR );
        strokeWeight( 1 );

        x = region.renderStart() + round( binding_site.relativeStart()*region.pixelPerBP() );
        w = max( round( binding_site.len()*region.pixelPerBP() ), BINDING_SITE_MIN_WIDTH );
        h = round( (float)BINDING_SITE_HEIGHT * (binding_site.strength() - binding_site_type.minStrength())/binding_site_type.maxMinusMinStrength() );

        if ( _binding_site_rolled_over && (_rolled_over_region == i) && (_rolled_over_binding_site == j) )
        {
          rolled_bs_x = x;
          rolled_bs_w = w;
          rolled_bs_h = h;
        }
        else
          rect( x, y, w, -h );
      }

      // render the colored, not rolled over binding sites
      for ( int j = 0; j < region.numBindingSites(); j++ )
      {
        binding_site = region.bindingSite(j);
        binding_site_type = _binding_site_types[binding_site.type()];

        if ( !binding_site_type.show() ) continue;        
        if ( _binding_site_type_rolled_over && (_rolled_over_binding_site_type == binding_site.type()) ) continue;
        if ( binding_site_type.showColor() == DEFAULT_COLOR ) continue;

        // first, render the filled in binding site
        noStroke();
        fill( binding_site_type.showColor() );
        stroke( BINDING_SITE_OUTLINE_COLOR );
        strokeWeight( 1 );

        x = region.renderStart() + round( binding_site.relativeStart()*region.pixelPerBP() );
        w = max( round( binding_site.len()*region.pixelPerBP() ), BINDING_SITE_MIN_WIDTH );
        h = round( (float)BINDING_SITE_HEIGHT * (binding_site.strength() - binding_site_type.minStrength())/binding_site_type.maxMinusMinStrength() );

        if ( _binding_site_rolled_over && (_rolled_over_region == i) && (_rolled_over_binding_site == j) )
        {
          rolled_bs_x = x;
          rolled_bs_w = w;
          rolled_bs_h = h;
        }
        else
          rect( x, y, w, -h );
      }

      // render all the sites with the rolled over site type
      if ( _binding_site_type_rolled_over ) {
        for ( int j = 0; j < region.numBindingSites(); j++ )
        {
          binding_site = region.bindingSite(j);
          binding_site_type = _binding_site_types[binding_site.type()];

          if ( !binding_site_type.show() ) continue;        
          if ( !(_binding_site_order[_rolled_over_binding_site_type] == binding_site.type()) ) continue;

          // first, render the filled in binding site
          noStroke();
          fill( binding_site_type.showColor() );
          stroke( ROLLED_OVER_FEATURE_LINE_COLOR );
          strokeWeight( ROLLED_OVER_FEATURE_LINE_WEIGHT );

          x = region.renderStart() + round( binding_site.relativeStart()*region.pixelPerBP() );
          w = max( round( binding_site.len()*region.pixelPerBP() ), BINDING_SITE_MIN_WIDTH );
          h = round( (float)BINDING_SITE_HEIGHT * (binding_site.strength() - binding_site_type.minStrength())/binding_site_type.maxMinusMinStrength() );

          rect( x, y, w, -h );
        }
      }

      // render the rolled over binding site
      if ( _binding_site_rolled_over && (_rolled_over_region == i) )
      {
        // render the site
        binding_site = region.bindingSite(_rolled_over_binding_site);
        binding_site_type = _binding_site_types[binding_site.type()];

        stroke( ROLLED_OVER_BINDING_SITE_LINE_COLOR );
        strokeWeight( ROLLED_OVER_BINDING_SITE_LINE_WEIGHT );

        fill( binding_site_type.showColor() );

        rect( rolled_bs_x, y, rolled_bs_w, -rolled_bs_h );
      }

      // render the annotations
      for ( int j = 0; j < region.numFeatures(); j++ )
      {
        feature = region.feature(j);

        if ( _feature_rolled_over && (_rolled_over_region == i) && (_rolled_over_feature == j ) )
        {
          stroke( ROLLED_OVER_FEATURE_LINE_COLOR );
          strokeWeight( ROLLED_OVER_FEATURE_LINE_WEIGHT );
        }
        else
        {
          stroke( FEATURE_LINE_COLOR );
          strokeWeight( FEATURE_LINE_WEIGHT );
        }

        x = region.renderStart() + round( feature.relativeStart()*region.pixelPerBP() );
        line( x, y, x, y+FEATURE_HEIGHT );

        if ( feature.len() != 0 )
        {
          w = max( round( feature.len()*region.pixelPerBP() ), BINDING_SITE_MIN_WIDTH );
          line( x, y+FEATURE_HEIGHT, x+w, y+FEATURE_HEIGHT );
          line( x+w, y+FEATURE_HEIGHT, x+w, y );
        }
      }

      // render the region line
      stroke( REGION_LINE_COLOR );
      strokeWeight( REGION_LINE_WEIGHT );
      line( region.renderStart(), y, region.renderStop(), y );

      // render the region tag
      textFont( _pixel_font_8 );
      textAlign( RIGHT, CENTER );
      fill( REGION_TAG_TEXT_COLOR );
      text( region.tag(), /*region.renderStart()*/-REGION_TAG_OFFSET, y-BINDING_SITE_HEIGHT/2-2 );


      y += REGION_HEIGHT+REGION_OFFSET;
    }

    popMatrix();


    // render over anything outside of the view area, as well as the scroll bar
    if ( _y_scroll_present ) {

      fill( BACKGROUND_COLOR );
      noStroke();
      rect( -_region_tag_max_width, _regions_view_h, _regions_view_w+_region_tag_max_width+1, WINDOW_BORDER_WIDTH );
      rect( -_region_tag_max_width, 0, _regions_view_w+_region_tag_max_width+1, -(WINDOW_BORDER_WIDTH+SCALE_BAR_OFFSET) );

      stroke( REGION_VIEW_OUTLINE_COLOR );
      strokeWeight( 1 );
      line( -_region_tag_max_width, 0, _regions_view_w+REGION_SCROLLBAR_OFFSET, 0 );
      line( _regions_view_w+REGION_SCROLLBAR_OFFSET, 0, _regions_view_w+REGION_SCROLLBAR_OFFSET, _regions_view_h );
      line( -_region_tag_max_width, _regions_view_h, _regions_view_w+REGION_SCROLLBAR_OFFSET, _regions_view_h );

      // render the scrollbar

      x = _regions_view_w + REGION_SCROLLBAR_OFFSET - REGION_SCROLLBAR_W/2;
      y = _y_scroll_bar_start;
      fill( REGION_VIEW_OUTLINE_COLOR );

      if ( !_y_scroll_bar_rolled_over ) noStroke();
      else stroke( SCROLL_BAR_HIGHLIGHT_COLOR );
      rect( x, y, REGION_SCROLLBAR_W, _y_scroll_bar_h );
    }

    // render the tags for rolled over binding sites or features
    if ( _binding_site_rolled_over ) {
      binding_site = _regions[_rolled_over_region].bindingSite(_rolled_over_binding_site);    

      String s = "coords: " + binding_site.start() + " - " + binding_site.stop();
      textFont( _pixel_font_8 );
      textAlign( LEFT, BOTTOM );
      noStroke();
      fill( ROLL_OVER_TEXT_BOX_COLOR );
      rect( mouseXInRegionView()-_regions_origin[0]+4, mouseYInRegionView()-_regions_origin[1]-8, textWidth(s), -11 );
      fill( ROLL_OVER_TEXT_BOX_TEXT_COLOR );
      text( s, mouseXInRegionView()-_regions_origin[0]+4+1, mouseYInRegionView()-_regions_origin[1]-8 );
    }
    if ( _feature_rolled_over ) {
      feature = _regions[_rolled_over_region].feature(_rolled_over_feature);

      // render the tag
      if ( feature.len() == 0 ) {
        String s1 = "coord: " + feature.start();
        String s2 = feature.tag();
        textFont( _pixel_font_8 );
        textAlign( LEFT, BOTTOM );
        noStroke();
        fill( ROLL_OVER_TEXT_BOX_COLOR );
        if ( s2.length() == 0 ) {
          rect( mouseXInRegionView()-_regions_origin[0]+4, mouseYInRegionView()-_regions_origin[1]-8, textWidth(s1), -11 );
          fill( ROLL_OVER_TEXT_BOX_TEXT_COLOR );
          text( s1, mouseXInRegionView()-_regions_origin[0]+4+1, mouseYInRegionView()-_regions_origin[1]-8 );
        }
        else {
          rect( mouseXInRegionView()-_regions_origin[0]+4, mouseYInRegionView()-_regions_origin[1]-8, max(textWidth(s1), textWidth(s2)), -22 );
          fill( ROLL_OVER_TEXT_BOX_TEXT_COLOR );
          text( s2, mouseXInRegionView()-_regions_origin[0]+4+1, mouseYInRegionView()-_regions_origin[1]-19 );
          text( s1, mouseXInRegionView()-_regions_origin[0]+4+1, mouseYInRegionView()-_regions_origin[1]-8 );
        }
      }
      else {
        String s1 = "coords: " + feature.start() + " - " + feature.stop();
        String s2 = feature.tag();
        textFont( _pixel_font_8 );
        textAlign( LEFT, BOTTOM );
        noStroke();
        fill( ROLL_OVER_TEXT_BOX_COLOR );
        rect( mouseXInRegionView()-_regions_origin[0]+4, mouseYInRegionView()-_regions_origin[1]-8, max(textWidth(s1), textWidth(s2)), -22 );
        fill( ROLL_OVER_TEXT_BOX_TEXT_COLOR );
        text( s2, mouseXInRegionView()-_regions_origin[0]+4+1, mouseYInRegionView()-_regions_origin[1]-19 );
        text( s1, mouseXInRegionView()-_regions_origin[0]+4+1, mouseYInRegionView()-_regions_origin[1]-8 );
      }
    }

    // first, the scale bar
    y = -SCALE_BAR_OFFSET;
    strokeWeight( SCALE_BAR_LINE_WEIGHT );
    stroke( SCALE_BAR_COLOR );
    textFont( _pixel_font_8 );
    fill( SCALE_BAR_COLOR );
    switch ( _region_alignment )
    {
    case REGION_ALIGNMENT_LEFT:
      line( 0, y, _scale_pixel_length, y );
      line( 0, y, 0, y+SCALE_BAR_H );
      line( _scale_pixel_length, y, _scale_pixel_length, y+SCALE_BAR_H );
      textAlign( LEFT, TOP );
      text( _scale_tag, _scale_pixel_length+4, y-3 );
      break;
    case REGION_ALIGNMENT_CENTER:
      line( _regions_view_w/2-_scale_pixel_length/2, y, _regions_view_w/2+_scale_pixel_length/2, y );
      line( _regions_view_w/2-_scale_pixel_length/2, y, _regions_view_w/2-_scale_pixel_length/2, y+SCALE_BAR_H );
      line( _regions_view_w/2+_scale_pixel_length/2, y, _regions_view_w/2+_scale_pixel_length/2, y+SCALE_BAR_H );
      textAlign( LEFT, TOP );
      text( _scale_tag, _regions_view_w/2+_scale_pixel_length/2+4, y-3 );
      break;
    case REGION_ALIGNMENT_RIGHT:
      line( _regions_view_w, y, _regions_view_w-_scale_pixel_length, y );
      line( _regions_view_w, y, _regions_view_w, y+SCALE_BAR_H );
      line( _regions_view_w-_scale_pixel_length, y, _regions_view_w-_scale_pixel_length, y+SCALE_BAR_H );
      textAlign( RIGHT, TOP );
      text( _scale_tag, _regions_view_w-_scale_pixel_length-4, y-3 );
      break;
    } 


    // render the label and selector box
    x = -REGION_TAG_OFFSET;
    y = -REGION_TAG_VERT_OFFSET;
    textFont( _verdana_font_12 );
    textAlign( RIGHT, BOTTOM );
    fill( LABEL_COLOR );
    text( _region_label, x, y+2 );

    x -= (int)textWidth(_region_label) + DROP_DOWN_BOX_OFFSET + DROP_DOWN_BOX_W;
    noStroke();
    fill( DROP_DOWN_FILL_COLOR );
    if ( !_regions_dropped_down ) 
      triangle( x+DROP_DOWN_BOX_W/2, y, x+DROP_DOWN_BOX_W, y-DROP_DOWN_BOX_W, x, y-DROP_DOWN_BOX_W );
    else
    {
      triangle( x, y, x+DROP_DOWN_BOX_W, y, x+DROP_DOWN_BOX_W/2, y-DROP_DOWN_BOX_W );

      // render the drop down box
      x = _regions_dropbox_origin[0];
      y = _regions_dropbox_origin[1];
      fill( DROPBOX_FILL_COLOR );
      stroke( DROPBOX_LINE_COLOR );
      strokeWeight( 1 );
      rect( x, y, _regions_dropbox_w, _regions_dropbox_h );

      x += DROPBOX_BORDER;
      y += DROPBOX_BORDER + DROPBOX_LINE_H;
      textAlign( LEFT, BOTTOM );
      fill( DROPBOX_TEXT_COLOR );
      stroke( DROPBOX_TEXT_COLOR );
      for ( int i = 0; i < _regions.length; i++ )
      {
        region = _regions[i];

        if ( _regions_dropbox_highlighted && (_pressed_regions_dropbox == i) )
        {
          fill( HIGHLIGHTED_COLOR );
          stroke( HIGHLIGHTED_COLOR );
        }
        else
        {
          fill( DROPBOX_TEXT_COLOR );
          stroke( DROPBOX_TEXT_COLOR );
        }

        if ( _regions_dropdown_rolled_over && (_rolled_over_regions_dropdown == i) )
        {
          textFont( _pixel_font_8b );
          strokeWeight( 2 );
        }
        else
        {
          textFont( _pixel_font_8 );
          strokeWeight( 1 );
        }
        text( region.tag(), x+DROPBOX_RADIO_BUTTON_W+DROPBOX_RADIO_BUTTON_OFFSET, y+2 );

        if ( region.show() )
        {
          smooth();
          line( x+DROPBOX_RADIO_BUTTON_W/4, y-DROPBOX_RADIO_BUTTON_W/2, x+DROPBOX_RADIO_BUTTON_W/2, y );
          line( x+DROPBOX_RADIO_BUTTON_W/2, y, x+DROPBOX_RADIO_BUTTON_W, y-DROPBOX_RADIO_BUTTON_W );
          noSmooth();
        }

        y += DROPBOX_LINE_H + DROPBOX_LINE_OFFSET;
      }
    }

    popMatrix();

    //
    // render the selected color picker under the mouse
    //
    if ( _color_selected )
    {
      x = mouseX - _region_view_origin[0] + _color_selected_box_offset[0];
      y = mouseY - _region_view_origin[1] + _color_selected_box_offset[1];
      stroke( COLOR_PICKER_SELECTED_LINE_COLOR );
      strokeWeight( 1 );
      fill( SITE_COLOR[_selected_color] );
      rect( x, y, COLOR_PICKER_BOX_W, COLOR_PICKER_BOX_W );
    }
    else if ( _legend_site_color_selected )
    {
      x = mouseX - _region_view_origin[0] + _legend_site_color_box_offset[0];
      y = mouseY - _region_view_origin[1] + _legend_site_color_box_offset[1];
      stroke( COLOR_PICKER_SELECTED_LINE_COLOR );
      strokeWeight( 1 );
      fill( _binding_site_types[_binding_site_order[_selected_legend_site_color]].showColor() );
      rect( x, y, BINDING_SITE_LEGEND_BOX_W, BINDING_SITE_LEGEND_BOX_H );
    }
  }

  void renderAnAlignmentSelector( int alignment, int x, int y )
  {
    strokeWeight(1);
    stroke( ALIGNMENT_SELECTOR_LINE_COLOR );
    if ( _region_alignment == alignment ) fill( ALIGNMENT_SELECTOR_SELECTED_FILL_COLOR );
    else fill( ALIGNMENT_SELECTOR_FILL_COLOR );
    rect( x, y, ALIGNMENT_SELECTOR_BOX_W, ALIGNMENT_SELECTOR_BOX_W );

    y += (ALIGNMENT_SELECTOR_BOX_W - 12)/2;

    switch( alignment )
    {
    case REGION_ALIGNMENT_LEFT:
      x += (ALIGNMENT_SELECTOR_BOX_W - _max_alignment_selector_line_w)/2;
      for ( int i = 0; i < 5; i++ )
      {
        line( x, y, x+_alignment_selector_line_w[i], y );
        y += 3;
      }
      break;

    case REGION_ALIGNMENT_CENTER:
      x += (ALIGNMENT_SELECTOR_BOX_W - _max_alignment_selector_line_w)/2 + _max_alignment_selector_line_w/2;
      for ( int i = 0; i < 5; i++ )
      {
        line( x-_alignment_selector_line_w[i]/2, y, x+_alignment_selector_line_w[i]/2, y );
        y += 3;
      }
      break;

    case REGION_ALIGNMENT_RIGHT:
      x += (ALIGNMENT_SELECTOR_BOX_W - _max_alignment_selector_line_w)/2 + _max_alignment_selector_line_w;
      for ( int i = 0; i < 5; i++ )
      {
        line( x, y, x-_alignment_selector_line_w[i], y );
        y += 3;
      }
      break;

    case REGION_ALIGNMENT_STRETCH:
      x += (ALIGNMENT_SELECTOR_BOX_W - _max_alignment_selector_line_w)/2;
      for ( int i = 0; i < 5; i++ )
      {
        line( x, y, x+_max_alignment_selector_line_w, y );
        y += 3;
      }
      break;
    }
  }

  void mouseMovedInView( int mx, int my )
  {
    _binding_site_rolled_over = _feature_rolled_over = _binding_site_type_rolled_over = false;
    _site_types_dropdown_rolled_over = _regions_dropdown_rolled_over = false;
    _y_scroll_bar_rolled_over = false;

    // check if we are in the regions region
    if ( (mx >= _regions_origin[0]) && (mx <= _regions_origin[0]+_regions_view_w) && 
      (my >= _regions_origin[1]) && (my <= _regions_origin[1]+_regions_view_h) )
      overBindingSiteOrFeature( mx-_regions_origin[0], my-_regions_origin[1] );

    else if ( (mx > _regions_origin[0]+_regions_view_w+REGION_SCROLLBAR_OFFSET-REGION_SCROLLBAR_W/2) && (mx <= _regions_origin[0]+_regions_view_w+REGION_SCROLLBAR_OFFSET+REGION_SCROLLBAR_W/2) && 
      (my >= _regions_origin[1]) && (my <= _regions_origin[1]+_regions_view_h) )
      overYScrollBar( my-_regions_origin[1] );

    //    else if ( _regions_dropped_down && (mx >= _spatial_legend_origin[0]) && (mx < _spatial_legend_origin[0]+_spatial_legend_w) && 
    //      (my >= _spatial_legend_origin[1]) && (my <= _spatial_legend_origin[1]+_spatial_legend_h) )
    //      overRegionDropbox( mx-_spatial_legend_origin[0], my-_spatial_legend_origin[1] );

    else if ( _regions_dropped_down && (mx >= (_regions_origin[0]-_region_tag_max_width)) && (mx < _regions_origin[0]) && 
      (my >= _regions_origin[1]) && (my <= _regions_origin[1]+_regions_view_h) )
      overRegionDropbox( mx-_regions_origin[0], my-_regions_origin[1] );

    else if ( (mx >= _site_legend_origin[0]) && (mx <= _site_legend_origin[0]+_site_legend_w) &&
      (my >= _site_legend_origin[1]) && (my <= _site_legend_origin[1]+max(_site_legend_h, _site_types_dropbox_h)) )
      overSiteLegend( mx-_site_legend_origin[0], my-_site_legend_origin[1] );
  }

  void mousePressedInView( int mx, int my, boolean right_pressed )
  {
    _color_selected = _legend_site_color_selected = _clear_button_selected = false; 
    _regions_dropdown_pressed = _site_types_dropdown_pressed = false;
    _site_types_dropbox_pressed = _site_types_dropbox_highlighted = _site_types_dropbox_highlighted = _regions_dropbox_pressed = _regions_dropbox_highlighted = false;
    _y_scroll_bar_pressed = false;

    // check if we are in the alignment selector region
    if ( (mx >= _alignment_selector_origin[0]) && (mx <= _alignment_selector_origin[0]+_alignment_selector_w) && 
      (my >= _alignment_selector_origin[1]) && (my <= _alignment_selector_origin[1]+_alignment_selector_h) )
      overAlignmentSelector( mx-_alignment_selector_origin[0], my-_alignment_selector_origin[1] );

    else if ( (mx >= _site_legend_origin[0]) && (mx <= _site_legend_origin[0]+_site_legend_w) &&
      (my >= (_site_legend_origin[1]-BINDING_SITE_LEGEND_TEXT_OFFSET)) && 
      ((!_site_types_dropped_down && (my <= _site_legend_origin[1]+_site_legend_h)) ||
      ( _site_types_dropped_down && (my <= _site_legend_origin[1]+_site_types_dropbox_h))) )
      siteLegendPressed( mx-_site_legend_origin[0], my-_site_legend_origin[1], right_pressed );

    else if ( (mx >= _color_picker_origin[0]) && (mx <= _color_picker_origin[0]+_color_picker_w) &&
      (my >= _color_picker_origin[1]) && (my <= _color_picker_origin[1]+_color_picker_h) )
      colorPickerPressed( mx-_color_picker_origin[0], my-_color_picker_origin[1] );

    else if ( (mx >= (_regions_origin[0]-_region_tag_max_width)) && (mx <= _regions_origin[0]+_regions_view_w) &&
      (my >= (_regions_origin[1]-BINDING_SITE_LEGEND_TEXT_OFFSET)) && (my <= _regions_origin[1]+_regions_view_h) )
      regionsPressed( mx-_regions_origin[0], my-_regions_origin[1], right_pressed );

    else if ( (mx > _regions_origin[0]+_regions_view_w+REGION_SCROLLBAR_OFFSET-REGION_SCROLLBAR_W/2) && (mx <= _regions_origin[0]+_regions_view_w+REGION_SCROLLBAR_OFFSET+REGION_SCROLLBAR_W/2) && 
      (my >= _regions_origin[1]) && (my <= _regions_origin[1]+_regions_view_h) )
      pressedYScrollBar( my-_regions_origin[1] );

    //    else if ( (mx >= _spatial_legend_origin[0]) && (mx <= _spatial_legend_origin[0]+_spatial_legend_w) &&
    //      (my >= (_spatial_legend_origin[1]-BINDING_SITE_LEGEND_TEXT_OFFSET)) && (my <= _spatial_legend_origin[1]+_spatial_legend_h) )
    //      regionsPressed( mx-_spatial_legend_origin[0], my-_spatial_legend_origin[1], right_pressed );

    if ( _site_types_dropdown_pressed )
      _site_types_dropped_down = !_site_types_dropped_down;

    if ( _regions_dropdown_pressed )
      _regions_dropped_down = !_regions_dropped_down;
  }

  void mouseDraggedInView( int mx, int my ) {
    _binding_site_type_rolled_over = false;

    if ( _y_scroll_bar_pressed ) yScrollBarDragged( my-_regions_origin[1] );

    else if ( (mx >= _site_legend_origin[0]) && (mx <= _site_legend_origin[0]+_site_legend_w) &&
      (my >= _site_legend_origin[1]) && (my <= _site_legend_origin[1]+max(_site_legend_h, _site_types_dropbox_h)) )
      overSiteLegend( mx-_site_legend_origin[0], my-_site_legend_origin[1] );
  }


  void mouseReleasedInView( int mx, int my )
  {
    //_y_scroll_bar_rolled_over = 
    _y_scroll_bar_pressed = false;

    if ( _legend_site_color_selected )
      legendSiteColorReleased( mx-_site_legend_origin[0], my-_site_legend_origin[1] );

    else if ( _clear_button_selected )
    {
      for ( int i = 0; i < _binding_site_types.length; i++ )
        _binding_site_types[i].showColor( DEFAULT_COLOR );
    }

    else if ( _site_types_dropbox_pressed )
    {
      _binding_site_types[_binding_site_order[_pressed_site_types_dropbox]].show(!_binding_site_types[_binding_site_order[_pressed_site_types_dropbox]].show());
      computeLegendParameters();
    }

    else if ( _regions_dropbox_pressed )
    {
      _regions[_pressed_regions_dropbox].show(!_regions[_pressed_regions_dropbox].show());
      setAlignmentAndScaleParameters();
      computeTotalRegionsHeight();
    }

    else if ( (mx >= _site_legend_origin[0]) && (mx <= _site_legend_origin[0]+_site_legend_w) && 
      (my >= _site_legend_origin[1]) && (my <= _site_legend_origin[1]+_site_legend_h) )
      if ( _color_selected )
        colorReleasedOverSiteLegend( mx-_site_legend_origin[0], my-_site_legend_origin[1], SITE_COLOR[_selected_color] );

    _color_selected = _legend_site_color_selected = _clear_button_selected = false;
    _regions_dropdown_pressed = _site_types_dropdown_pressed = false;
    _site_types_dropbox_pressed = /*_site_types_dropbox_highlighted = */_regions_dropbox_pressed = false;
  }

  boolean overYScrollBar( int my ) {
    _y_scroll_bar_rolled_over = false;
    if ( (my >= _y_scroll_bar_start) && (my <= _y_scroll_bar_start+_y_scroll_bar_h) ) _y_scroll_bar_rolled_over = true;
    return _y_scroll_bar_rolled_over;
  }

  void pressedYScrollBar( int my ) {
    if ( !_y_scroll_bar_rolled_over ) return;

    _y_scroll_bar_pressed = true;
    _y_scroll_bar_pressed_y = my;
  }

  void yScrollBarDragged( int my ) {
    int diff = my - _y_scroll_bar_pressed_y;

    if ( diff > 0 ) diff = min( diff, _regions_view_h - (_y_scroll_bar_start+_y_scroll_bar_h) );
    else diff = max( diff, -_y_scroll_bar_start );

    _y_scroll_bar_start += diff;
    _percent_regions_start = (float)_y_scroll_bar_start / (float)_regions_view_h;

    _y_scroll_bar_pressed_y = my;
  }

  boolean overBindingSiteOrFeature( int mx, int my )
  {
    int amt_scrolled = (int)(_percent_regions_start*_regions_h);
    my += amt_scrolled;

    //    int num_regions_skipped = floor( (float)amt_scrolled / (float)(REGION_HEIGHT+REGION_OFFSET) );
    //    int num_regions_ignored = max(0, floor( (float)(_regions_h - (amt_scrolled+_regions_view_h)) / (float)(REGION_HEIGHT+REGION_OFFSET) ));
    //    
    //    pushMatrix();
    //    translate( 0, -amt_scrolled );

    // determine which region the y position is in
    int index = 0;
    int y = 0;
    int num_active_regions = 0;
    for ( int j = 0; j < _regions.length; j++ )
    {
      if ( !_regions[j].show() ) continue;

      ++num_active_regions;
      if ( (my >= y) && (my <= y+REGION_HEIGHT) )
      {
        index = j;
        j = _regions.length;
      }
      y += REGION_HEIGHT + REGION_OFFSET;
    }
    int i = floor( (float)my/(float)(REGION_HEIGHT+REGION_OFFSET) );
    if ( i >= num_active_regions ) return false;

    // determine if it is in the space of a feature or binding site or offset
    int diff = my - i*(REGION_HEIGHT+REGION_OFFSET);  
    boolean in_binding_site_space = false;  
    if ( diff > REGION_HEIGHT ) return false;
    else if ( diff <= BINDING_SITE_HEIGHT ) in_binding_site_space = true;

    Region region = _regions[index];
    int x, w, h;

    if ( in_binding_site_space )
    {
      BindingSite binding_site;
      BindingSiteType binding_site_type;

      // go through the binding sites
      for ( int j = region.numBindingSites()-1; j >= 0; j-- )
      {
        binding_site = region.bindingSite(j);
        binding_site_type = _binding_site_types[binding_site.type()];

        if ( !binding_site_type.show() ) continue;

        // get the location information
        x = region.renderStart() + round( binding_site.relativeStart()*region.pixelPerBP() );
        w = max( round( binding_site.len()*region.pixelPerBP() ), BINDING_SITE_MIN_WIDTH );

        if ( (mx >= x) && (mx <= x+w) )
        {
          h = round( (float)BINDING_SITE_HEIGHT * (binding_site.strength() - binding_site_type.minStrength())/binding_site_type.maxMinusMinStrength() );
          if ( BINDING_SITE_HEIGHT-diff <= h )
          {
            _binding_site_rolled_over = true;
            _rolled_over_binding_site = j;
            _rolled_over_region = index;
          }
          return _binding_site_rolled_over;
        }
        else continue;
      }
    }
    else
    {
      Feature feature;

      // go through the features 
      for ( int j = 0; j < region.numFeatures(); j++ )
      {
        feature = region.feature(j);

        // get the location information
        x = region.renderStart() + round( feature.relativeStart()*region.pixelPerBP() );
        w = max( round( feature.len()*region.pixelPerBP() ), BINDING_SITE_MIN_WIDTH );

        if ( (mx >= x-1) && (mx <= x+w+1) )
        {
          _feature_rolled_over = true;
          _rolled_over_feature = j;
          _rolled_over_region = index;

          return _feature_rolled_over;
        }
      }
    }
    return false;
  }

  boolean overRegionDropbox( int mx, int my )
  {
    int x = _regions_dropbox_origin[0];
    int y = _regions_dropbox_origin[1]+DROPBOX_BORDER;

    for ( int i = 0; i < _regions.length; i++ )
    {
      if ( (mx >= x) && (mx <= x+_regions_dropbox_w) && (my >= y) && (my <= y+DROPBOX_LINE_H) )
      {
        _regions_dropdown_rolled_over = true;
        _rolled_over_regions_dropdown = i;

        return true;
      }
      y += DROPBOX_LINE_H + DROPBOX_LINE_OFFSET;
    }
    return false;
  }

  boolean overAlignmentSelector( int mx, int my )
  {
    int x = ALIGNMENT_SELECTOR_BOX_W;
    for ( int i = REGION_ALIGNMENT_LEFT; i <= REGION_ALIGNMENT_STRETCH; i++ )
    {
      if ( mx <= x )
      {
        _region_alignment = i;
        setAlignmentAndScaleParameters();
        return true;
      }
      x += ALIGNMENT_SELECTOR_BOX_W;
    }
    return false;
  }

  boolean overSiteLegend( int mx, int my )
  {
    int x, y;

    // if the site drop box is down, check for rollover there
    if ( _site_types_dropped_down )
    {
      x = _site_types_dropbox_origin[0];
      y = _site_types_dropbox_origin[1]+DROPBOX_BORDER;

      for ( int i = 0; i < _binding_site_types.length; i++ )
      {
        if ( (mx >= x) && (mx <= x+_site_types_dropbox_w) && (my >= y) && (my <= y+DROPBOX_LINE_H) )
        {
          _site_types_dropdown_rolled_over = true;
          _rolled_over_site_types_dropdown = i;

          return true;
        }
        y += DROPBOX_LINE_H + DROPBOX_LINE_OFFSET;
      }
      return false;
    }

    // check for rollover of the legend
    x = 0;
    y = BINDING_SITE_LEGEND_BORDER_W;

    BindingSiteType binding_site_type;
    for ( int i = 0; i < _binding_site_types.length; i++ )
    {
      binding_site_type = _binding_site_types[_binding_site_order[i]];

      if ( !binding_site_type.show() ) continue;

      if ( (mx >= x) && (mx <= x+_site_legend_w) && (my >= y) && (my <= y+BINDING_SITE_LEGEND_BOX_H) )
      {
        _binding_site_type_rolled_over = true;
        _rolled_over_binding_site_type = i;
        return true;
      }

      y += BINDING_SITE_LEGEND_BOX_OFFSET+BINDING_SITE_LEGEND_BOX_H;
    }

    return false;
  }

  boolean regionsPressed( int mx, int my, boolean right_pressed )
  {
    int x, y;

    // check if the regions dropdown is pressed
    textFont( _verdana_font_12 );
    //x = (int)textWidth("region") + DROP_DOWN_BOX_OFFSET;
    //x = _spatial_legend_w - DROP_DOWN_BOX_W;
    x = -((int)textWidth(_region_label) + DROP_DOWN_BOX_W + DROP_DOWN_BOX_OFFSET + REGION_TAG_OFFSET);
    y = -BINDING_SITE_LEGEND_TEXT_OFFSET-REGION_TAG_VERT_OFFSET;

    if ( (mx >= x) && (mx <= -REGION_TAG_OFFSET) && (my >= y) && (my <= 0) )
    {
      _regions_dropdown_pressed = true;
      return true;
    }

    // if the region drop box is down, check for pressed there
    if ( _regions_dropped_down )
    {
      x = _regions_dropbox_origin[0];
      y = _regions_dropbox_origin[1]+DROPBOX_BORDER;

      for ( int i = 0; i < _regions.length; i++ )
      {
        if ( (mx >= x) && (mx <= x+_regions_dropbox_w) && (my >= y) && (my <= y+DROPBOX_LINE_H) )
        {
          if ( !right_pressed )
            _regions_dropbox_pressed = true;
          else
            _regions_dropbox_highlighted = true;
          _pressed_regions_dropbox = i;

          return true;
        }
        y += DROPBOX_LINE_H + DROPBOX_LINE_OFFSET;
      }
      return false;
    }

    return false;
  }

  boolean colorPickerPressed( int mx, int my )
  {    
    int x = 0;
    int y = 0;

    for ( int i = 0; i < (NUM_SITE_COLORS/2); i++ )
    { 
      if ( (my >= y) && (my <= y+COLOR_PICKER_BOX_W) && (mx >= x) && (mx <= x+COLOR_PICKER_BOX_W) )
      {
        _color_selected = true;
        _selected_color = i;

        _color_selected_box_offset[0] = x - mx;
        _color_selected_box_offset[1] = y - my;

        return true;
      }
      x += (COLOR_PICKER_BOX_W + COLOR_PICKER_BOX_OFFSET );
    }

    if ( !_color_picker_one_row )
    {
      x = 0;
      y += (COLOR_PICKER_BOX_W + COLOR_PICKER_BOX_OFFSET );
    }

    for ( int i = (NUM_SITE_COLORS/2); i < NUM_SITE_COLORS; i++ )
    { 
      if ( (my >= y) && (my <= y+COLOR_PICKER_BOX_W) && (mx >= x) && (mx <= x+COLOR_PICKER_BOX_W) )
      {
        _color_selected = true;
        _selected_color = i;

        _color_selected_box_offset[0] = x - mx;
        _color_selected_box_offset[1] = y - my;

        return true;
      }
      x += (COLOR_PICKER_BOX_W + COLOR_PICKER_BOX_OFFSET );
    }

    // check if the clear button is pressed
    x = 0;
    y += (COLOR_PICKER_BOX_W + COLOR_PICKER_BUTTON_OFFSET );
    textFont( _pixel_font_8 );
    if ( (mx >= x) && (mx <= (x+textWidth("clear")+4)) && (my >= y) && (my <= y+COLOR_PICKER_BOX_W) )
    {
      _clear_button_selected = true;
      return true;
    }

    return false;
  }

  boolean siteLegendPressed( int mx, int my, boolean right_pressed )
  {
    int x, y;

    // check if the site types dropdown is pressed
    textFont( _verdana_font_12 );
    int w = (int)textWidth(_type_label) + DROP_DOWN_BOX_OFFSET + DROP_DOWN_BOX_W;
    x = 0;
    y = -BINDING_SITE_LEGEND_TEXT_OFFSET;
    if ( (mx >= x) && (mx <= x+w) && (my >= y) && (my <= 0) )
    {
      _site_types_dropdown_pressed = true;
      return true;
    }

    // if the site drop box is down, check for rollover there
    if ( _site_types_dropped_down )
    {
      x = _site_types_dropbox_origin[0];
      y = _site_types_dropbox_origin[1]+DROPBOX_BORDER;

      for ( int i = 0; i < _binding_site_types.length; i++ )
      {
        if ( (mx >= x) && (mx <= x+_site_types_dropbox_w) && (my >= y) && (my <= y+DROPBOX_LINE_H) )
        {
          if ( !right_pressed )
            _site_types_dropbox_pressed = true;
          else
            _site_types_dropbox_highlighted = true;

          _pressed_site_types_dropbox = i;

          return true;
        }
        y += DROPBOX_LINE_H + DROPBOX_LINE_OFFSET;
      }
      return false;
    }

    // check for the site types color boxes being pressed
    x = BINDING_SITE_LEGEND_BORDER_W;
    y = BINDING_SITE_LEGEND_BORDER_W;

    BindingSiteType binding_site_type;
    for ( int i = 0; i < _binding_site_types.length; i++ )
    {
      binding_site_type = _binding_site_types[_binding_site_order[i]];

      if ( !binding_site_type.show() ) continue;

      if ( (mx >= x) && (mx <= x+BINDING_SITE_LEGEND_BOX_W) && (my >= y) && (my <= y+BINDING_SITE_LEGEND_BOX_H) )
      {
        if ( binding_site_type.showColor() == DEFAULT_COLOR ) return false;

        _legend_site_color_selected = true;
        _selected_legend_site_color = i;

        _legend_site_color_box[0] = x;
        _legend_site_color_box[1] = y;
        _legend_site_color_box_offset[0] = x - mx;
        _legend_site_color_box_offset[1] = y - my;

        return true;
      }

      y += BINDING_SITE_LEGEND_BOX_OFFSET+BINDING_SITE_LEGEND_BOX_H;
    }

    return false;
  }

  boolean colorReleasedOverSiteLegend( int mx, int my, color c )
  {
    int x = BINDING_SITE_LEGEND_BORDER_W;
    int y = BINDING_SITE_LEGEND_BORDER_W;

    BindingSiteType binding_site_type;
    for ( int i = 0; i < _binding_site_types.length; i++ )
    {
      binding_site_type = _binding_site_types[_binding_site_order[i]];

      if ( !binding_site_type.show() ) continue;

      if ( (mx >= x) && (mx <= x+BINDING_SITE_LEGEND_BOX_W) && (my >= y) && (my <= y+BINDING_SITE_LEGEND_BOX_H) )
      {
        _binding_site_types[_binding_site_order[i]].showColor( c );
        return true;
      }

      y += BINDING_SITE_LEGEND_BOX_OFFSET+BINDING_SITE_LEGEND_BOX_H;
    }

    return false;
  }

  boolean legendSiteColorReleased( int mx, int my )
  {
    // first check if we have dropped it over another legend box
    colorReleasedOverSiteLegend( mx, my, _binding_site_types[_binding_site_order[_selected_legend_site_color]].showColor() );

    int x_diff = abs( (mx+_legend_site_color_box_offset[0])-_legend_site_color_box[0] );
    int y_diff = abs( (my+_legend_site_color_box_offset[1])-_legend_site_color_box[1] );
    if ( (x_diff > BINDING_SITE_LEGEND_BOX_W) || (y_diff > BINDING_SITE_LEGEND_BOX_H) )
      /*if ( ((mx <= _legend_site_color_box[0]) || (mx >= _legend_site_color_box[0]+BINDING_SITE_LEGEND_BOX_W)) ||
       ((my <= _legend_site_color_box[1]) || (my >= _legend_site_color_box[1]+BINDING_SITE_LEGEND_BOX_W)) )*/
    {
      _binding_site_types[_binding_site_order[_selected_legend_site_color]].showColor( DEFAULT_COLOR );
      return true;
    }
    return false;
  }

  void upKeyPressed()
  {
    if ( _site_types_dropped_down && _site_types_dropbox_highlighted )
    {
      int i = _pressed_site_types_dropbox-1;
      if ( i < 0 ) return;

      int bst = _binding_site_order[i];
      _binding_site_order[i] = _binding_site_order[_pressed_site_types_dropbox];
      _binding_site_order[_pressed_site_types_dropbox] = bst;

      _pressed_site_types_dropbox = i;
    }
    else if ( _regions_dropped_down && _regions_dropbox_highlighted )
    {
      int i = _pressed_regions_dropbox-1;
      if ( i < 0 ) return;

      Region r = _regions[i];
      _regions[i] = _regions[_pressed_regions_dropbox];
      _regions[_pressed_regions_dropbox] = r;

      _pressed_regions_dropbox = i;
    }
  }
  void downKeyPressed()
  {
    if ( _site_types_dropped_down && _site_types_dropbox_highlighted )
    {
      int i = _pressed_site_types_dropbox+1;
      if ( i == _binding_site_types.length ) return;

      int bst = _binding_site_order[i];
      _binding_site_order[i] = _binding_site_order[_pressed_site_types_dropbox];
      _binding_site_order[_pressed_site_types_dropbox] = bst;


      _pressed_site_types_dropbox = i;
    }
    else if ( _regions_dropped_down && _regions_dropbox_highlighted )
    {
      int i = _pressed_regions_dropbox+1;
      if ( i == _regions.length ) return;

      Region r = _regions[i];
      _regions[i] = _regions[_pressed_regions_dropbox];
      _regions[_pressed_regions_dropbox] = r;

      _pressed_regions_dropbox = i;
    }
  }
}

