import processing.pdf.*;

import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.util.Comparator;
import java.util.Collections;


int _w, _h, _min_w, _min_h;
boolean _print_pdf = false;
int _pdf_counter = 0;

RegionViewer _region_view;
int[] _region_view_origin;
int _region_view_width, _region_view_height;

void setup() 
{ 
  // to capture the size of the monitor
  _w = screen.width;
  _h = screen.height-40;

  // but lets just hardcode this for now
  _w = min( _w, 1200);
  _h = min( _h, 200);
  
  // read in the data
  readConfigFile( "config.txt" );

  // generate the fonts
  println( "Generating fonts..." );
  _pixel_font_8 = createFont( "PFTempestaSeven", 8, false );
  _pixel_font_8b = createFont( "PFTempestaSeven-Bold", 8, false );
  _verdana_font_12 = createFont( "Verdana", 12, true );
  _verdana_font_16 = createFont( "Verdana", 16, true );
  _verdana_font_30 = createFont( "Verdana", 30, true );
  
  // create the views
  _region_view = new RegionViewer();
  _region_view_origin = new int[2];

  // min width and height
  _min_w = 350;
  //_min_h = 400;
  _min_h = _region_view._total_legend_h + 2*WINDOW_BORDER_WIDTH;
  
  _w = max( _w, _min_w );
  _h = max( _h, _min_h );
  
  setDimensions();

  size( _w, _h );

  

  

  // set up for window resizing
  frame.setResizable(true);
  frame.addComponentListener(new ComponentAdapter() {
    public void componentResized(ComponentEvent e) {
      if(e.getSource()==frame) 
      { 
        _w = max( frame.getWidth(), _min_w*2 );
        _h = max( frame.getHeight(), _min_h+22 );
        frame.setSize(_w,_h); 
        _h -= 22; // offset on mac
        setDimensions();
      }
    }
  } 
  );
}

void setDimensions()
{
  // set up the REGION VIEW
  _region_view_origin[0] = WINDOW_BORDER_WIDTH;
  _region_view_origin[1] = WINDOW_BORDER_WIDTH;
  _region_view_width = _w - 2*WINDOW_BORDER_WIDTH;
  _region_view_height = _h - 2*WINDOW_BORDER_WIDTH;
  _region_view.setDimensions( _region_view_width, _region_view_height );
}

void draw() 
{ 
  // this is for rendering to a pdf file
  if ( _print_pdf ) {
    String filename = "output_" + nf(_pdf_counter++,2) + ".pdf";
    beginRecord(PDF, filename);
  }

  background( BACKGROUND_COLOR ); 

  // translate to the origin of the REGION VIEW
  pushMatrix();
  translate( _region_view_origin[0], _region_view_origin[1] );
  _region_view.render();
  popMatrix();

  // for turning off rendering to a pdf file
  if ( _print_pdf ) {
    _print_pdf = false;
    endRecord();
  }
}

void keyPressed() 
{
  if ( key == 'P' ) _print_pdf = true;
  
  else if (key == CODED) {
    if (keyCode == UP)
      _region_view.upKeyPressed();
    else if (keyCode == DOWN)
      _region_view.downKeyPressed();
  }
}

void mouseMoved()
{
  _region_view._binding_site_rolled_over = _region_view._feature_rolled_over = false;
  if ( (mouseX >= _region_view_origin[0]) && (mouseX <= _region_view_origin[0]+_region_view_width) &&
    (mouseY >= _region_view_origin[1]) && (mouseY <= _region_view_origin[1]+_region_view_height) )
  {
    _region_view.mouseMovedInView( mouseX-_region_view_origin[0], mouseY-_region_view_origin[1] );
  }
}

void mousePressed()
{
  if ( (mouseX >= _region_view_origin[0]) && (mouseX <= _region_view_origin[0]+_region_view_width) &&
    (mouseY >= _region_view_origin[1]) && (mouseY <= _region_view_origin[1]+_region_view_height) )
  {
    _region_view.mousePressedInView( mouseX-_region_view_origin[0], mouseY-_region_view_origin[1], mouseButton == RIGHT );
  }
}

void mouseReleased()
{
  if ( (mouseX >= _region_view_origin[0]) && (mouseX <= _region_view_origin[0]+_region_view_width) &&
    (mouseY >= _region_view_origin[1]) && (mouseY <= _region_view_origin[1]+_region_view_height) )
  {
    _region_view.mouseReleasedInView( mouseX-_region_view_origin[0], mouseY-_region_view_origin[1] );
  }
}

void mouseDragged()
{
  if ( (mouseX >= _region_view_origin[0]) && (mouseX <= _region_view_origin[0]+_region_view_width) &&
    (mouseY >= _region_view_origin[1]) && (mouseY <= _region_view_origin[1]+_region_view_height) )
  {
    _region_view.mouseDraggedInView( mouseX-_region_view_origin[0], mouseY-_region_view_origin[1] );
  }
}

